
/***************************************************************************
 *
 * IPAT - Interferometer Threshold Curve Plotter
 *
 * Stephen R. Whiteley (stevew@srware.com)
 * 
 * Copyright (c) 2002 Whiteley Research Inc.  All Rights Reserved. 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************
 * $Id: print.cc,v 2.8 2009/01/11 21:07:38 stevew Exp $
 ***************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifdef WIN32_NATIVE
#include <windows.h>
#include <windowsx.h>
#else
#include <gtk/gtk.h>
#ifdef WIN32
#include <windows.h>
#endif
#endif
#include "ipat.h"

//
// PostScript print interface
//

// US Letter defaults for Postscript
#define Uxoff 18
#define Uyoff 18
#define Uwidth  576
#define Uheight  756

#define RESOL 150 

// how big the font is in points=1/72 inch
#define FONTPTS 10

struct sGText {
    int x;
    int y;
    int xform;
    char *text;
    struct sGText *next;
};

struct printscr : public screen
{
    void clear() { }
    void define_color(int, int, int, int) { }
    void set_color(int) { }
    void pixel(int, int);
    void line(int, int, int, int);
    void box(int, int, int, int);
    void text(const char*, int, int, int);
    void set_linestyle(int);
    void input(int*, int*, int*, int*) { }

    bool open(char*);
    void close();
    void dump();

    int bytpline;
    int maxx;
    int maxy;
    char *base;
    struct sGText *textlist;
    unsigned char linestyle;
    unsigned char linestyle_stored;
    int fontwidth;
    int fontheight;
    int numcolors;
    int numlinestyles;
    FILE *plotfile;
};


void
printscr::pixel(int x, int y)
{
    unsigned char c = 0x80 >> (x & 7);
	*(base + ((x >> 3) + (maxy-y)*bytpline)) |= c;
}


#define ror(x,n) ((x >> n)  | (x << (8-n)))
#define swap(a,b) {int t=a; a=b; b=t;}

void
printscr::line(int x1, int y1, int x2, int y2)
{
	if (x2 < x1) {
		swap(x1,x2);
		swap(y1,y2);
	}
	int dx = x2 - x1;

	int next = bytpline;
	int dy = y1 - y2;

	int lcnt = maxy - y1;
	char *rgen = base + (x1 >> 3) + lcnt*next;
	unsigned char left = 0x80 >> ((lcnt+x1) & 7);
	unsigned char right = linestyle;

	if (y1 < y2) {
		next = -next;
		dy = -dy;
	}
	int dy2 = dy;

	unsigned char cbuf = 0x80 >> (x1 & 7);

	int errterm = 0;
    for (dy++; dy; dy--) {
		errterm += dx;
		if (errterm <= 0) {
			if (left & right) *rgen |= cbuf;
			rgen += next;
			left = ror(left,1);
			continue;
		}
		while (errterm > 0 && x1 != x2) {
			if (left & right) *rgen |= cbuf;
			left = ror(left,1);
			cbuf = ror(cbuf,1);
			if (cbuf & 0x80) rgen++;
			x1++;
			errterm -= dy2;
		}
		rgen += next;
		left = ror(left,1);
	}
}


void
printscr::box(int xl, int yl, int xu, int yu)
{
    if (yu < yl) swap(yu,yl);
    if (xu < xl) swap(xu,xl);

    // expand box so that it covers a box formed of lines with the
    // same coordinates
    //
    xu++;
    yl--;

    if (xl < 0)
        xl = 0;
    else if (xl > maxx)
        return;

    if (xu > maxx)
        xu = maxx;
    else if (xu < 0)
        return;

    if (yl < 0)
        yl = 0;
    else if (yl > maxy)
        return;

    if (yu > maxy)
        yu = maxy;
    else if (yu < 0)
        return;

    int lnum = (maxy - yu);
    int dy = yu - yl;

    unsigned char left  =  (0xff >> (xl & 7));
    unsigned char right = ~(0xff >> (xu & 7));
    int dx = (xu >> 3) - (xl >>= 3) - 1;
    if (dx < 0) { left &= right; dx = 0; right = 0; }
    char *rgen = base + xl + (long) lnum*bytpline;
    int next = bytpline - 1 - dx;

    unsigned char cbuf = 0xff;

    while (dy--) {
        *rgen |= left & cbuf;
        rgen++;
        int tx = dx;
        while (tx--) {
            *rgen |= cbuf;
            rgen++;
        }
        *rgen |= right & cbuf;
        rgen += next;
    }
}


void
printscr::set_linestyle(int linestyleid)
{
    if (linestyleid)
        linestyle = linestyle_stored;
    else
        linestyle = 0xff;
}


void
printscr::text(const char *text, int x, int y, int xform)
{
    sGText *gpt = new sGText;
    gpt->x = x;
    gpt->y = y;
    gpt->xform = xform;
    gpt->text = strdup(text);
    gpt->next = textlist;
    textlist = gpt;
}


bool
printscr::open(char *name)
{
    plotfile = fopen(name, "w");
    if (!plotfile) {
        return (true);
    }

    // 8" X 10.5" drawable area
    maxx = 8*RESOL - 1;
    maxy = 10*RESOL + RESOL/2 - 1;

    // bytes per line
    int bpline = maxx/8 + 1;

    base = new char [(maxy+1)*bpline];
    memset(base,0,(maxy+1)*bpline);

    bytpline = bpline;
    linestyle = 0xff;
    linestyle_stored = 0xcc;
    textlist = NULL;
    fontheight = FONTPTS*RESOL/72;
    fontwidth = 2*fontheight/3;
    numcolors = 2;
    numlinestyles = 2;
    return (false);
}


void
printscr::close()

{
    dump();
    sGText *tn;
    for (sGText *t = textlist; t; t = tn) {
        tn = t->next;
        delete [] t->text;
        delete t;
    }
    textlist = 0;
    delete [] base;
    base = 0;
    fclose(plotfile);
}



static char hexc[16] =
    { '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};

void
printscr::dump()
{
    int w = maxx + 1;
    int h = maxy + 1;

    // start file off with a %
    fprintf(plotfile, "%%! ipat plot\n");

    // set up a reasonable font
    fprintf(plotfile, "/Helvetica findfont %d scalefont setfont\n",
        (int)(FONTPTS*((double)w/Uwidth)));

    fprintf(plotfile, "/pixbuf %d string def\n",bytpline);
    fprintf(plotfile, "%d %d translate\n",Uxoff,Uyoff);
    fprintf(plotfile, "%f %f scale\n",(double)Uwidth/w,(double)Uheight/h);
    fprintf(plotfile, "%d %d 1\n",w,h);
    fprintf(plotfile, "matrix\n");
    fprintf(plotfile, "{currentfile pixbuf readhexstring pop}\nimage\n");

    // write data

    char *ptr = base + maxy*bytpline;
    int cnt = 0;
    for (int j = maxy; j >= 0; j--) {
        for (int i = 0; i < bytpline; i++) {
            unsigned char c = ~*(unsigned char*)ptr; // reverse black/white
            putc(hexc[c>>4],plotfile);
            putc(hexc[c&0xf],plotfile);
            ptr++;
            cnt++;
            if (cnt > 38) {
                cnt = 0;
                putc('\n',plotfile);
            }
        }
        ptr -= bytpline << 1;
    }
    fprintf(plotfile, "\n");

    // now for the text
    char tbuf[512];
    for (sGText *t = textlist; t; t = t->next) {
        int x = t->x;
        int y = t->y;

        char *s = tbuf;
        char *text = t->text;
        while (*text) {
            if (*text == '(' || *text == ')' || *text == '\\')
                *s++ = '\\';
            *s++ = *text++;
        }
        *s = '\0';
        fprintf(plotfile,"gsave\n");
        fprintf(plotfile,"%d %d translate\n",x,y);
        if (t->xform & 12)
            fprintf(plotfile,"%d %d scale\n",
                (t->xform & 8) ? -1 : 1, (t->xform & 4) ? -1 : 1);
        int deg = (t->xform & 3) * 90;
        if (deg)
            fprintf(plotfile,"%d rotate\n",deg);
        fprintf(plotfile,"0 0 moveto\n");
        fprintf(plotfile,"(%s) show\n",tbuf);
        fprintf(plotfile,"grestore\n");
    }
    fprintf(plotfile, "showpage\n");
}


void
panel::printscreen()
{
    printscr psc;
    if (psc.open(tofilename))
        return;
    psc.set_color(1);
    int fheight = psc.fontheight;
    int fwidth  = psc.fontwidth;
    int offset  = 3*psc.maxy/8;
    psc.W   = fwidth;
    psc.H   = fheight;
    psc.W2  = fwidth/2;
    psc.H2  = fheight/2;
    psc.H3  = psc.H + psc.H2;
    psc.XL  = 4*fwidth;
    psc.YL  = offset + 3*fheight;
    psc.XS  = psc.maxx;
    psc.YS  = offset + psc.maxy/2;
    psc.X   = psc.XS - psc.XL - 6*fwidth;
    psc.Y   = psc.YS - psc.YL - 3*fheight;
    psc.XC  = psc.XL + psc.X/2;
    psc.YC  = psc.YL + psc.Y/2;
    psc.XU  = psc.XL + psc.X;
    psc.YU  = psc.YL + psc.Y;

    diag_x = 100;
    diag_y = 100;
    psc.diagram();
    diag_x = 0;
    diag_y = 0;
    psc.displ();
    char buf[128];
    sprintf(buf,  "IPAT utility v%s   Whiteley Research Inc.,  wrcad.com",
        VERSION);
    psc.text(buf,psc.XL,psc.YU + 5*psc.H, 0);
    psc.close();

    if (!tofile && cmdtext && *cmdtext) {
#ifdef WIN32
        const char *err = msw_RawFileToPrinter(cmdtext, tofilename);
        if (err) {
            char buf[256];
            sprintf(buf,
                "Sorry, but Windows reports that the print job failed:\n%s",
                err);
            MessageBox(0, buf, "Warning", MB_ICONWARNING);
        }
#else
        sprintf(buf, "%s %s", cmdtext, tofilename);
        system(buf);
#endif
        unlink(tofilename);
    }
}


#ifdef WIN32

int
msw_ListPrinters(int *curprinter, char ***printers)
{
    int numprinters = 0;
    *printers = 0;

    // list available printers

    DWORD sz, num;
    EnumPrinters(PRINTER_ENUM_CONNECTIONS | PRINTER_ENUM_LOCAL,
        0, 4, 0, 0, &sz, &num);
    BYTE *bf = new BYTE[sz];
    if (EnumPrinters(PRINTER_ENUM_CONNECTIONS | PRINTER_ENUM_LOCAL,
            0, 4, bf, sz, &sz, &num) && num) {
        PRINTER_INFO_4 *pi = (PRINTER_INFO_4*)bf;
        numprinters = num;
        *printers = new char*[num];
        for (unsigned i = 0; i < num; i++)
            (*printers)[i] = strdup(pi[i].pPrinterName);
    }
    delete [] bf;
    char buf[256];
    if (GetProfileString("windows", "device", ",,,", buf, 256) > 0) {
        strtok(buf, ",");
        for (unsigned i = 0; i < num; i++) {
            if (!strcmp((*printers)[i], buf)) {
                *curprinter = i;
                break;
            }
        }
    }
    return (numprinters);
}


const char *
msw_RawFileToPrinter(const char *szPrinterName, const char *szFileName)
{
    HANDLE     hPrinter;
    DOC_INFO_1 DocInfo;
    DWORD      dwJob;
    DWORD      dwBytesWritten;
    BYTE       buf[8192];

    // Need a handle to the printer.
    if ( ! OpenPrinter((char*)szPrinterName, &hPrinter, NULL ) )
        return ("can't open printer");

    // Fill in the structure with info about this "document."
    char stupid_microsoft_1[32];
    strcpy(stupid_microsoft_1, "My Document");
    DocInfo.pDocName = stupid_microsoft_1;
    DocInfo.pOutputFile = NULL;
    char stupid_microsoft_2[8];
    strcpy(stupid_microsoft_2, "RAW");
    DocInfo.pDatatype = stupid_microsoft_2;
    // Inform the spooler the document is beginning.
    if ( (dwJob = StartDocPrinter( hPrinter, 1, (BYTE*)&DocInfo )) == 0 ) {
        ClosePrinter( hPrinter );
        return ("start document call failed");
    }
    // Start a page.
    if ( ! StartPagePrinter( hPrinter ) ) {
        EndDocPrinter( hPrinter );
        ClosePrinter( hPrinter );
        return ("start page call failed");
    }
    // Send the data to the printer.
    FILE *fp = fopen(szFileName, "rb");
    if ( ! fp) {
        EndPagePrinter( hPrinter );
        EndDocPrinter( hPrinter );
        ClosePrinter( hPrinter );
        return ("can't open file");
    }
    for (;;) {
        DWORD n = fread(buf, 1, 8192, fp);
        if (n == 0)
            break;
        if ( ! WritePrinter( hPrinter, buf, n, &dwBytesWritten ) ||
                dwBytesWritten != n ) {
            EndPagePrinter( hPrinter );
            EndDocPrinter( hPrinter );
            ClosePrinter( hPrinter );
            return ("data transfer error");
        }
    }
    fclose(fp);

    // End the page.
    if ( ! EndPagePrinter( hPrinter ) ) {
        EndDocPrinter( hPrinter );
        ClosePrinter( hPrinter );
        return ("page end call failed");
    }
    // Inform the spooler that the document is ending.
    if ( ! EndDocPrinter( hPrinter ) ) {
        ClosePrinter( hPrinter );
        return ("document end call failed");
    }
    // Tidy up the printer handle.
    ClosePrinter( hPrinter );
    return (0);
}

#endif


//
// The help text
//

const char *help_text =
"IPAT - Interferometer Threshold Curve Plotter\n"
"\n"
"This program plots the interference patterns of two and three\n"
"Josephson junction interferometers under the assumption that the\n"
"junctions are perfect RSJ with no coupled flux.\n"
"\n"
"Within these assumptions, the computation is exact, to within an\n"
"iteration tolerance of .1 percent.  The algorithms used have been\n"
"discussed in the literature, and are implemented in an efficient\n"
"manner which provides rapid computation.\n"
"\n"
"--- Running IPAT\n"
"\n"
"From a command line, type \"ipat\".  Any words following \"ipat\" on the\n"
"command line are expected to be a path to a file in the format of the\n"
"output file from Jspice3/WRspice operating range analysis.  The pass\n"
"or fail points from the file will be plotted along with the\n"
"interference pattern (see Load Points below).  A window will appear,\n"
"containing some input areas, and graphical areas for the plot and a\n"
"schematic.  There is a menu bar along the top, containing a \"WR\"\n"
"button, and File, Options, and Help menus.\n"
"\n"
"If an \".ipatrc\" file is found in the current or home directories, the\n"
"the values found in the file are used to set some program defaults. \n"
"This file has a very simple format, as seen in the example below.\n"
"\n"
"------- Example .ipatrc file\n"
"\n"
"# This is the startup file for IPAT.  It contains three entries:  the\n"
"# number of junctions to start with (2 or 3), the control line\n"
"# coupling constant, and the printer string, IN THAT ORDER.  This will\n"
"# provide the initial values for these parameters.  A line\n"
"# with \'#\' in the first column is a comment and is ignored.  Arbitrary\n"
"# text can follow the numbers.  If the third entry is blank (i.e.,\n"
"# contains one or more spaces, the print operation will by default\n"
"# create an \"ipat_plot.ps\" file.\n"
"#\n"
"# Unless you want to change these defaults, don\'t bother installing\n"
"# .ipatrc.  Otherwise, copy this file to your home directory or the\n"
"# current directory, and rename it to \".ipatrc\".\n"
"#\n"
"2           Start with two junctions\n"
"1.0         Assume perfect coupling (entry must be > 0)\n"
"lpr -h\n"
"\n"
"-------\n"
"\n"
"The gtk user interface provides a set of entry widgets for\n"
"specification of component values, and a toolbar that contains menu\n"
"entries for various functions to be described.  The display is\n"
"automatically updated after each change.\n"
"\n"
"The File menu contains the following selections:\n"
"\n"
"  Save\n"
"\n"
"    The Save button in the File menu brings up a dialog asking for a\n"
"    name to save the current data set under.  This name can be any\n"
"    name, but it can not start with the character \'#\', and it can\'t\n"
"    include white space.  The data set will be saved in the file\n"
"    \".ipat.data\", in the user\'s home directory.  If the home directory\n"
"    can\'t be determined, such as under Windows if the environment\n"
"    variable $HOME is not defined to a directory path, a file named\n"
"    ipat.data in the current directory will be used.  In either case\n"
"    the file will be created if it does not already exist.  The format\n"
"    is very simple:  each line contains the name, followed by the\n"
"    number of junctions, followed by the nine values from the input\n"
"    entry widgets, top to bottom as on-screen.  Note that some of\n"
"    these values are not really used for two junctions, but some\n"
"    numeric value must be present in that position as a place-holder. \n"
"    If the given name is already present, the new data set will\n"
"    overwrite the old.\n"
"\n"
"  Recall\n"
"\n"
"    Pressing the Recall button in the File menu will produce a menu\n"
"    containing the names of all saved parameter sets.  Selecting one of\n"
"    the entries will load those values into ipat, and update the display.\n"
"\n"
"  Delete\n"
"\n"
"    The Delete button in the File menu will prompt for a dataset name to\n"
"    delete from storage.  The response should be the name of a parameter\n"
"    set previously stored.\n"
"\n"
"  Load Points\n"
" \n"
"    This allows a \"points\" file to be read.  The points found in the\n"
"    file will be added to the plot.  The file format is the\n"
"    Jspice3/WRspice operating range output format.\n"
"\n"
"  Print\n"
"\n"
"    This brings up a pop-up which controls printing the current\n"
"    display.\n"
"\n"
"  Quit\n"
"\n"
"    Exit IPAT.\n"
"\n"
"The Options menu contains the following selections:\n"
"\n"
"  Three Junctions\n"
"\n"
"    When set, a three-junction device will be plotted.  When unset, a\n"
"    two-junction device will be shown.\n"
"\n"
"  Phase Plot\n"
"\n"
"    When set, a phase plot will be presented.  When unset, a threshold\n"
"    plot will be presented.\n"
"\n"
"  Show Negative X\n"
"\n"
"    When set, both positive and negative control currents will be\n"
"    shown.  When unset, only positive control currents will be shown.\n"
"\n"
"\n"
"  Show Negative Y\n"
"\n"
"    When set, both positive and negative threshold voltages will be\n"
"    shown.  When unset, only positive threshold voltages will be\n"
"    shown.\n"
"\n"
"  Suppress Axes\n"
"\n"
"    When set, the axes and grid points will not be shown.\n"
"\n"
"  Suppress Secondary\n"
"\n"
"    When set, the secondary contours of three-junction devices will not\n"
"    be shown.\n"
"\n"
"  Suppress Tertiary\n"
"\n"
"    When set, the tertiary contours of three-junction devices will not\n"
"    be shown.\n"
"\n"
"  Plot Fail Points\n"
"\n"
"    If plotting points from a file (Load Points in File Menu) show the\n"
"    fail points.\n"
"\n"
"  Show Marker\n"
"\n"
"    Enables a marker and printout of the cursor coordinates.  Retire\n"
"    the marker by pressing any key.  The left mouse button marks a\n"
"    location which is used as a reference for the coordinate printing,\n"
"    which can be deleted with the right button (i.e., go back to\n"
"    absolute coordinates).\n"
"\n"
"The Help menu contains the following selections:\n"
"\n"
"  Help\n"
"\n"
"    Bring up a window containing a a description of the program\n"
"    features.\n"
"\n"
"\n"
"--- Theory\n"
"\n"
"Two-junction solution:\n"
"\n"
"  Basic equations:\n"
"\n"
"  (1)  It = Ic1*sin(phi1) + Ic2*sin(phi2)\n"
"       (gate current)\n"
"\n"
"  (2)  FLX = phi1 - phi2 + PHI*( L1*Ic1*sin(phi1)  -\n"
"           L2*Ic2*sin(phi2) )\n"
"       (external flux)\n"
"\n"
"  (3)  1/(Ic1*cos(phi1)) + 1/(Ic2*cos(phi2) + PHI*(L1+L2) = 0\n"
"       (The auxiliary condition)\n"
"       This expression can be derived by introducing a Lagrange\n"
"       multiplier m, and setting I~ = It + m*FLX.  Equating\n"
"       dI~/dphi1 = 0 and dI~/dphi2 = 0 yields (3).\n"
"\n"
"  where\n"
"    \n"
"    It   threshold gate current\n"
"    FLX  externally applied flux\n"
"     \n"
"  There are two junctions, one with phase phi1 and critical current\n"
"  Ic1, and similar for the second junction.  The loop inductance is\n"
"  split between L1 and L2, whth the gate current driving the\n"
"  connection point.  L1 connects to J1, and L2 connects to J2.\n"
"\n"
"  The algorithm is to simply loop over phi1, then again over phi2 to\n"
"  better fill in the curve, using (3) to get the other angle. \n"
"  Equations (1) and (2) provide a locus of points which define the\n"
"  threshold curve.\n"
"\n"
"\n"
"Three-junction solution:\n"
"\n"
"  Basic equations:\n"
"  sN = IcN*sin(phiN)\n"
"  cN = icN*cos(phiN)\n"
"\n"
"  (1)  It = s1 + s2 + s3\n"
"          = Il + Ir, Il/It = C  ( C is splitting fraction )\n"
"       (gate current)\n"
"\n"
"  (2)  f1 = phi1 - phi2 + PHI*(L1*s1 - L2*(C(s2 + s3) + (C-1)*s1)) + Order1\n"
"       (left loop flux)\n"
"\n"
"       f2 = phi2 - phi3 + PHI*(L3*((1-C)*(s1 + s2) - C*s3) - l4*s3) + Order2\n"
"       (right loop flux)\n"
"\n"
"  (3)  I~ = It + A*(f1 terms) + B*(f2 terms)\n"
"       (lagrange multipliers)\n"
"\n"
"  (4)  Partial derivatives\n"
"\n"
"       phi1:\n"
"       c1 +\n"
"          A * (1 + PHI*L1*c1 + (1-C)*PHI*L2*c1) +\n"
"          B * (1-C)*PHI*L3*c1 = 0\n"
"\n"
"       phi2:\n"
"       c2 -\n"
"          A * (1 + C*PHI*L2*c2) +\n"
"          B * (1 + (1-C)*PHI*L3*c2) = 0\n"
"\n"
"       phi3:\n"
"       c3 -\n"
"          A * C*PHI*L2*c3 -\n"
"          B * (1 + PHI*L4*c3 + C*PHI*L3*c3) = 0\n"
"\n"
"  (5)  FLX = f1 + f2\n"
"       (total external flux)\n"
"\n"
"  (6)  (L3+L4)*f1 - (L1+L2)*f2 = 0\n"
"       (assume the flux is coupled through the loop inductance)\n"
"       A common control line divides the flux between the two loops as\n"
"       above.\n"
"\n"
"We loop over phi1, then phi2, and finally phi3, to ensure that the\n"
"curves are filled in.  Given one angle, (4) and (6) are used to find\n"
"the other two, using a Newton-Rhapson numerical solution algorithm. \n"
"(1) and (5) provide the locus of points which represent the threshold. \n"
"Before plotting, we discard any points where\n"
"\n"
"    PHI*Ic1*cos(phi1)*(L1+L2) < -1, or\n"
"    PHI*Ic2*cos(phi3)*(L3+L4) < -1.\n"
"\n"
"These points represent nonphysical solutions.  The above conditions\n"
"are not rigorously sufficient to eliminate all such solutions, but in\n"
"practice there does not seem to be a problem.\n"
"\n"
"This process is repeated with Order1 set to 2*PI to get the secondary\n"
"thresholds (where the two loops differ by one flux quantum), and\n"
"Order2 is set to -2*PI and the process is again repeated to get the\n"
"third order thresholds (loops differ by two flux quanta).\n"
"\n"
"The best way to learn IPAT, and to learn about interferometers, is to\n"
"experiment.  Have fun!\n"
"\n"
"--- References\n"
"\n"
"Won-Tien Tsang and T. Van Duzer, \"The analysis of parallel arrays of\n"
"two and three Josephson junctions,\" J. Appl. Phys., Vol. 46, No. 10,\n"
"pp. 4573-4580, October 1975.\n"
"\n"
"  Tsang and Van Duzer outline the Lagrange multiplier method of solving\n"
"  the interferometer equations, which is the method used in IPAT for\n"
"  three junction interferometers.\n"
"\n"
"R. L. Peterson and C. A Hamilton, \"Analysis of threshold curves for\n"
"superconducting interferometers,\" J. Appl. Phys., Vol. 50, No. 12,\n"
"pp.8135-8142, December 1979.\n"
"\n"
"  Peterson and Hamilton provide a useful approximate screening\n"
"  condition for determining which solutions are unstable.\n"
"\n"
"E. O. Schulz-DuBios and P. Wolf, \"Static Characteristics of\n"
"Josephson Interferometers,\" Appl. Phys. (Springer-Verlag), Vol. 16,\n"
"pp. 317-338, 1978.\n"
"\n"
"  Shulz-DuBois and Wolf provide detailed analysis of the stability\n"
"  criteria.\n"
"\n"
"Gregory S. Lee, \"A simple physical principle for determining mode\n"
"boundaries in superconducting loop circuits,\" J. Appl. Phys., Vol 66,\n"
"No. 6, 15 September 1989.\n"
"\n"
"  Lee shows an interesting connection between the small-signal\n"
"  model of an interferometer circuit and the general equations\n"
"  governing the threshold behavior.\n"
"\n";
